import java.util.ArrayList;
import java.util.StringTokenizer;
import java.sql.*;

/**
	Used to parse a "struct" directive and build the
	Insert into the database using the data provided.
*/

public class DStruct {

private StringBuffer buffer;
private String directive;
private String tableName;
private ArrayList fields;

/** Constructor with initialization */
public DStruct(String table, String directive) throws Exception {

	this.tableName=table;
	this.directive=directive;
	buffer=new StringBuffer("");
	fields=new ArrayList();

	try {
		parseDirective();
	} catch(Exception e) {
		System.err.println("mismatched table directive");
		e.printStackTrace();
		throw new Exception("mismatched table directive");
	}
}

/** parse the directive and build the SQL string */
private void parseDirective() throws Exception {

	String el;
	int i=0;

	/* first remove the "(", ")" and tabs */
	directive=directive.replace('\t',' ');
	directive=directive.trim();
	directive=removeMultiple(directive,' ',';');

	if(directive.endsWith(")")) 
		directive=directive.substring(0,directive.length()-1);

	/* now, I have all the single elements divided by ','
	   and the single fields divided by ';'. Let's parse! */
	buffer=new StringBuffer("INSERT INTO ");
	buffer.append(tableName);
	buffer.append(" (");

	StringTokenizer t=new StringTokenizer(directive,",");
	while(t.hasMoreTokens()) {
		FieldDescriptor d=new FieldDescriptor(t.nextToken());
		fields.add(d);
		buffer.append(d.getName());
		buffer.append(",");
	}

	buffer.deleteCharAt(buffer.length()-1);
	buffer.append(") VALUES (");
	for(i=0;i<fields.size();i++)
		buffer.append("?,");
	buffer.deleteCharAt(buffer.length()-1);
	buffer.append(")");

	System.err.println(buffer.toString());

}

/** remove multiple occurrence of a char */
private String removeMultiple(String s, char search, char subst)
{
	int pos=0;
	int len=s.length();
	char ch;
	boolean first=true;
	StringBuffer b=new StringBuffer("");

	for(pos=0;pos<len;pos++) {
		ch=s.charAt(pos);
		if(ch==search && first) {
			first=false;
			b.append(subst);
		}

		if(ch!=search) {
			first=true;
			b.append(ch);
		}
	}

	return b.toString();
}


/** return the directive as a SQL String */
public String toString() {
	return buffer.toString();
}

/** return a field descriptor from the list */
public FieldDescriptor getField(int num) {
	FieldDescriptor f;
	try {
		f=(FieldDescriptor) fields.get(num);
		return f;
	} catch(Exception e) {
		return null;
	}
}

/** return the nunmber of fields in the list */
public int getFieldCount() {
	return fields.size();
}

}
