import java.sql.Types;
import java.util.StringTokenizer;

/**
	Used to store and describe the position and other
	caracteristic of a single field.
*/

public class FieldDescriptor {

private String name;
private int position;
private int start;
private int end;
private int type;
private String value;
private String check;

/** constructor */
public FieldDescriptor(String name, int position, int type)
	throws Exception {
	this.name=name;
	this.position=position;
	this.start=-1;
	this.end=-1;
	this.type=type;
	value="";
	check="";
}

/** constructor */
public FieldDescriptor(String name, int start, int end, int type)
	throws Exception {
	this.name=name;
	this.start=start;
	this.end=end;
	this.type=type;
	this.position=-1;
	check="";
}

/** empty constructor without initialization */
public FieldDescriptor()
	throws Exception {
	this.name="noname";
	this.start=0;
	this.end=0;
	this.type=Types.INTEGER;
	this.position=0;
	check="";
}

/** constructor */
public FieldDescriptor(String name, int position, String type)
	throws Exception {
	this.name=name;
	this.position=position;
	this.type=checkType(type);
	this.start=-1;
	this.end=-1;
	check="";
}

/** constructor */
public FieldDescriptor(String name, int start, int end, String type)
	throws Exception {
	this.name=name;
	this.position=-1;
	this.type=checkType(type);
	this.start=start;
	this.end=end;
	check="";
}

/** get a string and convert to a SQL Type */
public int checkType(String type) {
	if(type.compareToIgnoreCase("integer")==0) 
		return java.sql.Types.INTEGER;
	if(type.compareToIgnoreCase("varchar")==0) 
		return java.sql.Types.VARCHAR;
	if(type.compareToIgnoreCase("char")==0) 
		return java.sql.Types.VARCHAR;
	if(type.compareToIgnoreCase("date")==0) 
		return java.sql.Types.DATE;
	if(type.compareToIgnoreCase("timestamp")==0) 
		return java.sql.Types.TIMESTAMP;
	if(type.compareToIgnoreCase("numeric")==0) 
		return java.sql.Types.NUMERIC;
	if(type.compareToIgnoreCase("float")==0) 
		return java.sql.Types.FLOAT;
	if(type.compareToIgnoreCase("decimal")==0) 
		return java.sql.Types.DECIMAL;
	return -1;

}

/** constructor */
public FieldDescriptor(String line) throws Exception {
	try {
		parseLine(line);
	} catch(Exception e) {
		System.err.println("error parsing "+line);
		e.printStackTrace();
	}
}

private void parseLine(String line) throws Exception {

	String p,y;
	StringTokenizer t=new StringTokenizer(line,";");

	name=t.nextToken();
	p=t.nextToken();
	y=t.nextToken();

	if(t.hasMoreTokens())
		check=t.nextToken();
	else
		check="";

	/* check position field */
	p=p.toUpperCase();
	if(p.startsWith("POS(")) {
		p=p.substring(4);
		p=p.substring(0,p.length()-1);
		position=Integer.parseInt(p);
	}

	if(p.startsWith("INT(")) {
			p=p.substring(4);
			p=p.substring(0,p.length()-1);
			t=new StringTokenizer(p,":-");
			start=Integer.parseInt(t.nextToken());
			end=Integer.parseInt(t.nextToken());
	}

	this.type=checkType(y);
}

/** return the name of the field */
public String getName() {
	return name;
}

/** return the position of the field */
public int getPosition() {
	return position;
}

/** return the start position of the field */
public int getStart() {
	return start;
}

/** return the end position of the field */
public int getEnd() {
	return end;
}

/** return the field's type */
public int getType() {
	return type;
}

/** set the value */
public void setValue(String value) {
	this.value=value;
}

/** get the value */
public String getValue() {
	return value;
}

/** check the value */
public boolean check() {

	if(check.length()==0)
		return true;

	if(check.compareTo("check")==0 && value.length()==0)
		return false;
	if(check.compareTo("check")!=0) {
		if(value.compareToIgnoreCase(check)==0 )
			return true;
		else
			return false;
	}
	return true;
}

}

