import java.sql.*;
import java.io.File;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.FileReader;
import java.util.StringTokenizer;
import java.util.ArrayList;
import java.text.SimpleDateFormat;

/**
	IMP_DB

	this is a generical utility used to read an ascii file and
	insert the data into a database, interpretating the information
	using a "control file".

	first version: 29/10/2001	by Davide Bianchi
*/

public class Impdb {

private String ctlfile;
private String url;
private String driver;
private String user;
private String pass;
private String opt;
private String datafile;
private char separator;
private char enclosed;
private String preaction;
private String postaction;
private String errfile;
private BufferedWriter errWriter;
private File errorFile;
private boolean skipfirst;

private SimpleDateFormat dateFormat;

private ArrayList structures;
private DStruct dstruct;

private Connection conn;
private PreparedStatement stmt;

/** Main method */
public static void main(String args[]) {
	Impdb impdb=new Impdb(args);
}

/** Display messages */
public void messages(int msgnum) {

	if(msgnum==1) 
		System.out.println("Version 1.0 - 29/10/2001");

	if(msgnum==8) {
		System.out.println("unrecognized option: '"+opt+"'");
	}

	if(msgnum==9) {
		System.out.print("usage: Impdb -f <ctlfile> ");
		System.out.print("[-u <username>] ");
		System.out.print("[-p <password>] ");
		System.out.print("[-U <database url>] ");
		System.out.println("[-d <driver>]");
	System.out.println("See the documentation for further details.");
	}

	if(msgnum==-99) {
		System.out.println("Driver:"+driver);
		System.out.println("User:"+user);
		System.out.println("Pass:"+pass);
		System.out.println("Url:"+url);
		System.out.println("Ctrl:"+ctlfile);
	}
}

/** Constructor: read command parameter */
public Impdb(String args[]) {

	int i=0;
	boolean processed=false;
	structures=new ArrayList();
	dstruct=null;

	skipfirst=false;
	preaction=null;
	postaction=null;

	for(i=0;i<args.length;i++) {

		opt=args[i];
		processed=false;

		if(args[i].compareTo("-h")==0 ||
		   args[i].compareTo("-help")==0) {
			messages(9);
			break;
		}

		if(args[i].compareTo("-v")==0 ||
		   args[i].compareTo("-version")==0) {
			messages(1);
			break;
		}

		if(args[i].compareTo("-u")==0 ||
		   args[i].compareTo("-user")==0) {
			user=args[++i];
			processed=true;
		}

		if(args[i].compareTo("-p")==0 ||
		   args[i].compareTo("-pass")==0) {
			pass=args[++i];
			processed=true;
		}

		if(args[i].compareTo("-U")==0 ||
		   args[i].compareTo("-url")==0) {
			url=args[++i];
			processed=true;
		}

		if(args[i].compareTo("-d")==0 ||
		   args[i].compareTo("-driver")==0) {
			driver=args[++i];
			processed=true;
		}

		if(args[i].compareTo("-f")==0 ||
		   args[i].compareTo("-file")==0) {
			ctlfile=args[++i];
			processed=true;
		}

		if(!processed) {
			messages(8);
			break;
		}
	}

	try {
		if(ctlfile != null) {
			readCtrlFile(ctlfile);
		} else {
			System.err.println("no control file specified");
			throw new Exception("no control file specified");
		}

		/* check if i have all the information */
		checkData();

		/* Ok, process the file */
		processFile();

	} catch(Exception e) {
	}
}

/** process the input file */
private void processFile() throws Exception {

	BufferedReader b=new BufferedReader(new FileReader(datafile));
	errWriter=new BufferedWriter(new FileWriter(errorFile));

	String line;
	int lnum=0;
	int tnum=0;
	int xnum=0;
	boolean ok=false;

	/* open the database connection */
	try {
		Class.forName(driver).newInstance();
	} catch(Exception e) {
		System.err.println("database driver "+driver+" not found");
		throw new Exception("database driver "+driver+" not found");
	}

	try {
		conn=DriverManager.getConnection(url,user,pass);
	} catch(Exception e) {
		System.err.println("unable to open a connection:"+
			e.getMessage());
		throw new Exception(e.getMessage());
	}

	try {
		if(preaction!=null) {
			Statement xx=conn.createStatement();
			xx.execute(preaction);
			xx.close();
		}
	} catch(Exception e) {
		System.err.println("pre-action failed:"+e.getMessage());
		throw new Exception(e.getMessage());
	}

	try {
		while((line=b.readLine())!=null) {
			tnum++;

			if((tnum==1 && skipfirst ) || line.length()==0) {
			} else {

				for(xnum=0;xnum<structures.size();xnum++) {
					dstruct=(DStruct) structures.get(xnum);
					stmt=conn.prepareStatement(
						dstruct.toString());
					lnum+=processLine(line);
					stmt.close();
				}
			}
		}
	} catch(Exception e) {}

	try {
		if(postaction!=null) {
			Statement xx=conn.createStatement();
			xx.execute(postaction);
			xx.close();
		}
	} catch(Exception e) {
		System.err.println("post-action failed:"+e.getMessage());
		throw new Exception(e.getMessage());
	}
	System.out.println(tnum+"/"+lnum);
	errWriter.flush();
	return;
}

/** process a single line */
private int processLine(String line) throws Exception {

	String s;
	SpecialTokenizer t=new SpecialTokenizer(line,separator,enclosed);
	int fcount=0;
	FieldDescriptor f;
	java.sql.Date d1;

	try {
		while(t.hasMoreTokens()) {

			fcount++;
			s=t.nextToken();

			if(s.startsWith("\"") || s.startsWith("'"))
				s=s.substring(1,s.length()-1);
			s=s.trim();

			for(int i=0;i<dstruct.getFieldCount();i++) {
				f=dstruct.getField(i);
				if(f.getPosition()==fcount) 
					f.setValue(s);
			}
		}

		/* now insert the data */
		for(int i=0;i<dstruct.getFieldCount();i++) {

			f=dstruct.getField(i);

			if(!f.check())
				return 0;

			switch(f.getType()) {
			case java.sql.Types.INTEGER:
				stmt.setInt(i+1,Integer.parseInt(f.getValue()));
				break;
			case java.sql.Types.CHAR:
				stmt.setString(i+1,f.getValue());
				break;
			case java.sql.Types.NUMERIC:
				stmt.setInt(i+1,Integer.parseInt(f.getValue()));
				break;
			case java.sql.Types.VARCHAR:
				stmt.setString(i+1,f.getValue());
				break;
			case java.sql.Types.TIMESTAMP:
				d1=new java.sql.Date(
				dateFormat.parse(f.getValue()).getTime());
				stmt.setDate(i+1,d1);
				break;
			case java.sql.Types.DATE:
				d1=new java.sql.Date(
				dateFormat.parse(f.getValue()).getTime());
				stmt.setDate(i+1,d1);
				break;
			default:
				System.err.println("unsupported type:"+
				f.getType());
				throw new Exception("usnupported type:"+
				f.getType());
			}
		}


	} catch(Exception e) {
		errWriter.write("error trying to parse "+line+":"+
		e.getMessage());
		System.err.println("error trying to parse "+line+":"+
		e.getMessage());
		return 0;
	}

	try {
		stmt.executeUpdate();
	} catch(Exception e) {
		errWriter.write("error executing the insert for "+line+":"+
		e.getMessage());
		System.err.println("error executing the insert for "+line+":"+
		e.getMessage());
		e.printStackTrace();
		System.exit(0);
		return 0;
	}

	return 1;
}

/** check the data */
private void checkData() throws Exception {

	if(datafile==null) {
	System.err.println("control file does not specify any datafile");
	throw new IOException("incomplete control file: no datafile");
	} else {
		File f=new File(datafile);
		if(!f.exists()) {
		System.err.println("datafile "+datafile+" not found");
		throw new IOException("datafile "+datafile+" not found");
		}
	}

	if(driver==null) {
		System.err.println("no database driver specified");
		throw new IOException("no database driver specified");
	}

	if(url==null) {
		System.err.println("no database url specified");
		throw new IOException("no database url specified");
	}

	if(dateFormat==null) {
		dateFormat=new SimpleDateFormat("DD-MM-yyyy");
	}

	if(errfile==null) {
		errfile=datafile+".err";
		errorFile=new File(errfile);
		if(errorFile.exists()) 
			errorFile.delete();
	}
}

/** Read the control file */
private void readCtrlFile(String ctrlfile)
	throws Exception {

	String s;
	String s1;
	String table="";
	boolean inTable=false;
	File cfile = new File(ctrlfile);
	StringBuffer directive=new StringBuffer("");

	if( ! cfile.exists() ) {
		System.err.println("control file "+ctrlfile+" does not exists");
		throw(new IOException("ctrlfilemissing"));
	}

	BufferedReader b=new BufferedReader(new FileReader(cfile));
	while((s=b.readLine()) != null ) {

		s1=s.toLowerCase();
		if(s.startsWith("#") || s.length()==0) {
			/* comments and blank lines are ignored */
		} else {
			if(s1.startsWith("table ")) {
				/* table directive */
				inTable=true;
				s=s.substring(5);
				if(s.endsWith(" ("))
					s=s.substring(1,s.length()-1);
				table=s;
				s="";
			}

			/** if not table, maybe directive ? */
			if(!inTable) 
				processParameter(s);

			/* join lines of the table directive */
			if(inTable) {
				directive.append(s);
			}

			/* end of table directive */
			if(s.startsWith(")") && inTable) {
				inTable=false;
				structures.add(
				new DStruct(table,directive.toString()));
			}
		}
	}

	return;
}

/** process parameters or directives */
private void processParameter(String s) throws Exception {

	StringTokenizer	t=new StringTokenizer(s,"=");
	String par, val;
	boolean ok=false;

	if(t.countTokens()<2) {
		System.err.println("unrecognized directive "+s);
		throw(new IOException("ctrlfileerror"));
	}

	par=t.nextToken().toLowerCase();
	val=t.nextToken();

	/* check the parameter */
	if(par.compareTo("url")==0 && url==null)  {
		ok=true;
		url=val;
	}

	if(par.compareTo("preaction")==0) {
		preaction=val;
		ok=true;
	}

	if(par.compareTo("postaction")==0) {
		postaction=val;
		ok=true;
	}

	if(par.compareTo("errfile")==0) {
		errfile=val;
		ok=true;
	}

	if(par.compareTo("user")==0 && user==null) {
		user=val;
		ok=true;
	}

	if(par.compareTo("pass")==0 && pass==null) {
		ok=true;
		pass=val;
	}

	if(par.compareTo("driver")==0 && driver==null) {
		driver=val;
		ok=true;
	}

	if(par.compareTo("infile")==0) {
		datafile=val;
		ok=true;
	}

	if(par.compareTo("separator")==0) {
		separator=val.charAt(0);
		ok=true;
	}

	if(par.compareTo("enclosed")==0) {
		enclosed=val.charAt(0);
		ok=true;
	}

	if(par.compareTo("dateformat")==0) {
		ok=true;
		try {
			dateFormat=new SimpleDateFormat(val);
		} catch(Exception e) {
			System.err.println("wrong date format:"+val);
			throw new Exception("wrong date format:"+val);
		}
	}

	if(par.compareTo("skipfirst")==0 ) {
		ok=true;
		if(val.compareTo("yes")==0) 
			skipfirst=true;
		else
			skipfirst=false;
	}


	if(!ok) {
		System.err.println("unrecognized token "+par);
		throw new Exception("Unrecognized token "+par);
	}
}
}
